/*********************************************************************
 * gpaco.h: header file for gpaco
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#ifndef __GPACO_H_
#define __GPACO_H_

#include "config.h"
#include "util.h"

#include <gtk/gtk.h>
#include <glib/gstdio.h>

#if ENABLE_NLS
#	include <glib/gi18n.h>
#else
#	define _(x)		x
#	define N_(x)	x
#endif

#define ISWRITABLE(dir) \
	(!paco_inpaths(dir, DEFAULT_EXCLUDE) && !access(dir, W_OK))

#define REFRESH_GUI \
	while (gtk_events_pending()) gtk_main_iteration()

#define SEPARATOR_MENU_ITEM(menu) \
	gtk_container_add(GTK_CONTAINER(menu), gtk_separator_menu_item_new())


/* Compression actions */
#define GP_GZIP        0
#define GP_BZIP2       1
#define GP_UNCOMPRESS  2


/* A file */
typedef struct {
	guint32 id;
	gchar *name;
	gint32 size;
	gboolean miss;
} file_t;

/* A package */
typedef struct {
	guint id;
	guint date;				/* Installation date */
	gchar *log;				/* Path to the log of the package */
	gchar *info;			/* Path to the info log of the package, or NULL */
	gchar *name;			/* Name of the package */
	gchar *icon;			/* Path to the icon of the package, or NULL */
	gfloat size_inst;		/* Installed size in bytes */
	gfloat size_miss;		/* Missing size in bytes */
	guint32 n_inst;			/* Number of installed files */
	guint32 n_miss;			/* Number of missing files */
	GSList *files;			/* Logged files */
	GtkWidget *window;		/* The window of the package */
	GtkWidget *list_tv;		/* The treeview for the list of files */
	int list_type;		/* Type of the files to be listed in page "Files" */
#define FILE_INSTALLED	(1 << 0)
#define FILE_MISSING	(1 << 1)
#define FILE_LOGGED		(FILE_INSTALLED | FILE_MISSING)
	GtkWidget *rm_textview;	/* The textview in the "Remove" page */
	int rm_flags;
#define RM_SHARED     (1 << 0)
#define RM_KEEP_LOGS  (1 << 1)
} pkg_t;

typedef struct {
	GtkWidget *window;
	gint width;
	gint height;
	gint x;
	gint y;
} window_t;

/* Columns in the main tree view */
enum {
	COL_ICON,
	COL_NAME,
	COL_SIZE,
	COL_MISSING_SIZE,
	COL_DATE,
	COL_FILES,
	COL_MISSING_FILES,
	COL_ID,
	NUM_COLS
};


/* Global variables */
extern window_t _window;		/* The main window */
extern GtkTreeModel *_model;	/* The model of the main window tree view */
extern GtkWidget *_bin_dialog;	/* The binary package creation dialog */
extern gchar *_logdir;			/* Current log directory */
extern gchar *_tmp;
extern GSList *_pkgs;			/* List of packages currently displayed */
extern GtkTooltips *_tips;		/* Tips for the whole application */
extern int _opt;			/* Display options */
#define SHOW_NAME             (1 << 0)
#define SHOW_SIZE             (1 << 1)
#define SHOW_MISSING_SIZE     (1 << 2)
#define SHOW_DATE             (1 << 3)
#define SHOW_FILES            (1 << 4)
#define SHOW_MISSING_FILES    (1 << 5)
#define SHOW_RULES            (1 << 6)
#define SHOW_TIPS             (1 << 7)
#define SHOW_DAY              (1 << 8)
#define SHOW_HOUR             (1 << 9)
#define SHOW_ICON             (1 << 10)
#define MISS_SIZE_AS_PERCENT  (1 << 11)
#define APPARENT_SIZE         (1 << 12)
extern gint _blksize;			/* Size units */
extern gboolean _updating;


/* Global functions */
extern void search();
extern gboolean update_pkg(pkg_t *);
extern void update_db();
extern GtkWidget *remove_page(pkg_t *);
extern gboolean gp_spawn(gchar **error, gchar *cmd);
extern gchar *gp_locale_to_utf8(gchar *);
extern gchar *gp_filename_to_utf8(gchar *);
extern gchar *gp_filename_from_utf8(gchar *);
extern GtkWidget *gp_button(const gchar *stock_id, const gchar *text,
  GCallback cb, gpointer cb_data);
extern GtkWidget *gp_menu(const gchar *label, GtkWidget *menubar);
extern GtkWidget *gp_menu_item(const gchar *title, gboolean sensitive,
  GtkWidget *menu, GCallback cb, gpointer cb_data);
extern GtkWidget *gp_check_menu_item(const gchar *title, gboolean active,
  GtkWidget *menu, GCallback cb, gpointer data);
extern GtkWidget *gp_image_menu_item(const gchar *stock_id, gchar *text,
  GtkWidget *menu, gboolean sensitive, GCallback cb, gpointer cb_data);
extern GtkWidget *gp_progress_dialog(char *msg, GtkWidget *window, GtkWidget **pbar);
extern void gp_error_dialog(gchar *msg, GtkWidget *parent, gchar *errmsg);
extern gint gp_question_dialog(GtkWidget *parent, gchar *msg);
extern void gp_column(const gchar *title, gint column_id, gboolean visible,
  gfloat xalign, GtkTreeCellDataFunc func, GtkWidget *treeview);
extern void gp_unlink(const gchar *path);
extern gchar *strsize(gfloat);
extern gint casesort(GtkTreeModel *, GtkTreeIter *, GtkTreeIter *, gpointer);
extern void binary_pkg(pkg_t *);
extern void files_treeview_update(pkg_t *);
extern GtkWidget *files_page(pkg_t *);
extern void main_window_new();
extern void main_list_set_pkg(GtkListStore *, GtkTreeIter *, pkg_t *);
extern void pkg_window_new(pkg_t *pkg, gint page_id);
extern gboolean destroy_pkg(pkg_t *);
extern pkg_t *new_pkg(gchar *name, gint *id);
extern void get_pkg_files(pkg_t *);
extern void free_pkg_data(pkg_t *, gpointer);
extern void preferences();
extern GtkWidget *properties_page(pkg_t *);


#endif  /* __GPACO_H_ */


/* vim: set ts=4 */
