/*********************************************************************
 * search.c - File search dialog.
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#include "gpaco.h"

static GtkWidget *search_dialog, *entry;


static void search_browse(GtkWidget *window)
{
	GtkWidget *chooser;
	G_CONST_RETURN gchar *filename;
	gchar *name, *name_8;
	gboolean ok;
	
	chooser = gtk_file_chooser_dialog_new(
	  _("Select file"), GTK_WINDOW(window), GTK_FILE_CHOOSER_ACTION_OPEN,
	  GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
	g_object_set(chooser, "show-hidden", TRUE, NULL);
	gtk_window_set_default_size(GTK_WINDOW(chooser), 450, 350);
	gtk_window_set_transient_for(GTK_WINDOW(chooser), GTK_WINDOW(window)); 
	
	ok = ((filename = gtk_entry_get_text(GTK_ENTRY(entry))) && *filename == '/');
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), ok ? filename : "/");
	
	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK) {
		name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
		name_8 = gp_filename_to_utf8(name);
		gtk_entry_set_text(GTK_ENTRY(entry), name_8);
		g_free(name);
		g_free(name_8);
	}
	gtk_widget_destroy(chooser);
}


static void search_file(GtkWidget *view)
{
	pkg_t *pkg;
	file_t *pkgfile;
	GtkTextBuffer *buf;
	GtkTextIter start, end;
	const gchar *file_8;
	gchar *file, *pkgname_8;
	GSList *p;
	register GSList *f;
	
	file_8 = gtk_entry_get_text(GTK_ENTRY(entry));
	g_return_if_fail(file_8 != NULL);
	file = gp_filename_from_utf8((gchar *)file_8);
	
    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	gtk_text_buffer_get_end_iter(buf, &end);
	gtk_text_buffer_get_start_iter(buf, &start);
	gtk_text_buffer_delete(buf, &start, &end);
	
	for (p = _pkgs; p; p = p->next) {
		pkg = (pkg_t *)(p->data);
		for (f = pkg->files; f; f = f->next) {
			pkgfile = (file_t *)(f->data);
			if (!strcmp(pkgfile->name, file)) {
				pkgname_8 = gp_filename_to_utf8(pkg->name);
				gtk_text_buffer_insert(buf, &end, pkgname_8, -1);
				gtk_text_buffer_insert(buf, &end, "\n", 1);
				g_free(pkgname_8);
				break;
			}
		}
	}
	g_free(file);
}


static void search_dialog_destroy()
{
	search_dialog = NULL;
}


void search()
{
	GtkWidget *hbox, *button, *scroll, *view;
	GtkBox *vbox;
	GtkButtonBox *bbox;
	
	if (search_dialog) {
		gtk_window_present(GTK_WINDOW(search_dialog));
		return;
	}

	search_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(search_dialog), _("Search file"));
	g_signal_connect(search_dialog, "destroy", G_CALLBACK(search_dialog_destroy), NULL);
	gtk_window_set_resizable(GTK_WINDOW(search_dialog), FALSE); 
	gtk_window_set_transient_for(GTK_WINDOW(search_dialog), GTK_WINDOW(_window.window));
	gtk_container_set_border_width(GTK_CONTAINER(search_dialog), 8);

	vbox = GTK_BOX(GTK_DIALOG(search_dialog)->vbox);
	hbox = gtk_hbox_new(FALSE, 6);
	entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);	
	button = gp_button(GTK_STOCK_OPEN, _("_Browse"), G_CALLBACK(search_browse), search_dialog);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_box_pack_start(vbox, hbox, FALSE, FALSE, 0);
	
	view = gtk_text_view_new();
    gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
    g_object_set(view, "left-margin", 4, "right-margin", 4, NULL);
	scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
	  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scroll), view);
	gtk_box_pack_start(vbox, scroll, TRUE, TRUE, 10);
	
	bbox = GTK_BUTTON_BOX(GTK_DIALOG(search_dialog)->action_area);
	gtk_button_box_set_layout(bbox, GTK_BUTTONBOX_EDGE);
	button = gp_button(GTK_STOCK_FIND, _("_Search"), G_CALLBACK(search_file), view);
	gtk_dialog_add_action_widget(GTK_DIALOG(search_dialog), button, GTK_RESPONSE_NONE);
	button = gtk_dialog_add_button(GTK_DIALOG(search_dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
	g_signal_connect_swapped(button, "clicked", G_CALLBACK(gtk_widget_destroy), search_dialog);

	gtk_widget_show_all(search_dialog);
}


/* vim: set ts=4 */
