/*********************************************************************
 * path.c - path manipulation routines.
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2004-2005 David Rosal i Ricart <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/
/* vim: set ts=4 */

#include "util.h"


/* Check whether 'path' matches any path in the colon-separated
 * 'list' of paths. */
int paco_inpaths(const char *path, const char *list)
{
	size_t n;
	char *p, *q, aux[PATH_MAX];
	
	if (!list || !*list || !path || !*path)
		return 0;

	strncpy(aux, list, PATH_MAX);
	
	for (p = q = aux; (p = q) && *p; ) {
		if (*p == ':')
			p++;
		if ((q = strchr(p, ':')))
			*q++ = '\0';
		n = strlen(p);
		/* strip trailing slashes */
		while (n > 1 && *(p + n - 1) == '/')
			*(p + --n) = '\0';
		if (!strcmp(p, "/")) /* case "/" */
			return 1;
		else if (!strncmp(path, p, n) && (path[n] == '\0' || path[n] == '/'))
			return 1;
	}

	return 0;
}


/*
 * "Absolutize", and resolve '//', '/./' and '/../' occurrences, in 'path',
 * writing the result in 'real'.
 * If 'resolve_dir_symlinks' is 1, intermediate directory symlinks are
 * resolved, otherwise symlinks are not resolved at all.
 *
 * Return the lenght of 'real', or -1 on error.
 */
int paco_realpath(const char *path, char *real, int resolve_dir_symlinks)
{
	char dir[PATH_MAX], aux[PATH_MAX], *p;
	int j = 0;
	
	if (*path == '/')
		p = (char *)path;
	else {
		if (!getcwd(dir, PATH_MAX))
			return -1;
		snprintf(aux, PATH_MAX, "%s/%s", dir, path);
		p = aux;
	}

	while (*p) {
		if (!strncmp(p, "//", 2))
			p++;
		else if (!strncmp(p, "/./", 3) || !strncmp(p, "/.", 3))
			p += 2;
		else if (!strncmp(p, "/../", 4)) {
			while (--j && real[j] != '/');
			p += 3;
		}
		else
			real[j++] = *p++;
	}
	
	/* Remove trailing slash */
	if (real[j - 1] == '/')
		j--;
	real[j] = '\0';
	
	if (!resolve_dir_symlinks)
		return j;
		
	strcpy(dir, real);
	
	if ((p = strrchr(&dir[1], '/'))) {
		*p++ = '\0';
		if (realpath(dir, aux))
			j = snprintf(real, PATH_MAX, "%s/%s", aux, p);
	}
	
	return j;
}
