/*********************************************************************
 * update.c - Update the logs of the packages
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2004-2005 David Rosal i Ricart <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#include "paco.h"


static void rebuild_line(char *old, char *new, size_t size)
{
	char *p;
	size_t i = 0;

	for (p = old; *p && i < size - 4; ) {
		if (!strncmp(p, "|-|", 3)) {
			memcpy(new + i, "|-2", 3);
			i += 3;
			p += 3;
		}
		else if (!strncmp(p, "|-", 3)) {
			memcpy(new + i, "|-2", 4);
			return;
		}
		else if (!strncmp(p, "|@|", 3)) {
			memcpy(new + i, "|-1", 3);
			i += 3;
			p += 3;
		}
		else if (!strncmp(p, "|@", 3)) {
			memcpy(new + i, "|-1", 4);
			return;
		}
		else
			*(new + i++) = *p++;
	}
	
	*(new + i) = '\0';
}


static void rebuild_log(char *log)
{
	FILE *f, *ftmp;
	char buf[KILOBYTE], aux[KILOBYTE];

	if (!(f = fopen(log, "r")))
		return;
	else if (!(ftmp = fopen(_tmp, "w"))) {
		fclose(f);
		return;
	}
	
	while (paco_fgets(buf, sizeof(buf), f)) {
		rebuild_line(buf, aux, sizeof(aux));
		fprintf(ftmp, "%s\n", aux);
	}

	fclose(f);
	fclose(ftmp);

	paco_rename(_tmp, log);
}


static int islogged(char *my_pkg, list_t *pkgs)
{
	list_t *p;
	pkg_t *pkg;
	
	for (p = pkgs; p; p = p->next) {
		pkg = (pkg_t *)(p->data);
		if (!strcmp(pkg->name, my_pkg))
			return 1;
	}
	return 0;
}


/* Remove info logs for non-logged pkgs */
static void remove_obsolete_info_logs(list_t *pkgs)
{
	struct dirent *d;
	DIR *dir;
	char *infodir, *infolog;
	int errors = 0;
	
	xasprintf(&infodir, "%s/_info", _logdir);
	
	if (!(dir = opendir(infodir))) {
		free(infodir);
		return;
	}
	
	debug(0, _("Removing obsolete info logs... "));
	
	while ((d = readdir(dir))) {
		if (d->d_name[0] == '.' || islogged(d->d_name, pkgs))
			continue;
		xasprintf(&infolog, "%s/%s", infodir, d->d_name);
		if (!unlink(infolog)) {
			say("%s ", infolog);
			rewinddir(dir);
		}
		else {
			say_errno("unlink(\"%s\")", infolog);
			errors++;
		}
		free(infolog);
	}
	
	closedir(dir);
	free(infodir);

	if (!errors)
		debug(0, _("ok\n"));
}


void do_update(char *args[])
{
	list_t *pkgs, *p;
	pkg_t *pkg;
	int rebuild = 0;
	char *stamp, buf[64];
	FILE *f;
	
	if (access(_logdir, W_OK) < 0)
		die_errno("access(\"%s\")", _logdir);

	xasprintf(&stamp, "%s/.updated", _logdir);
	
	if (!(f = fopen(stamp, "r")))
		rebuild++;
	else {
		if (!fgets(buf, sizeof(buf), f) || strcmp(buf, PACO_VERSION))
			rebuild++;
		fclose(f);
	}
		
	pkgs = get_pkgs(args);
	
	for (p = pkgs; p; p = p->next) {
		
		if (!(pkg = (pkg_t *)(p->data)))
			continue;
		
		if (rebuild)
			rebuild_log(pkg->log);
		
		say(_("Updating %s... "), pkg->log);
		
		if (paco_update(pkg->log, _tmp, _apparent) == PACO_ERROR_EMPTY_LOG)
			say(_("empty log (removed)\n"));
		else
			say(_("ok\n"));
	}
	
	if (args == ALL_PACKAGES) {
		remove_obsolete_info_logs(pkgs);
		paco_touch(stamp);
	}
	
	pkg_list_free(pkgs);
	free(stamp);
}


/* vim: set ts=4: */
